program AutoCollect ;
{ Timed automatic frame aquisition to disk or memory }
{ Copyright  2000, Digital Optics Ltd }


{ ******** DESCRIPTION ********

  The AutoCollect module is a time lapse image capture application that enables
  the user to capture images automatically at a regular interval and either save
  them to disk individually or append them to a sequence in memory.
  
  The module works with both PVCAM cameras and video frame grabbers. You must
  select the device type (using the Setup command) before starting a capture
  sequence.
  
  While acquiring images, the latest frame is displayed on screen in a window.
  
  The module can also maintain a log file which records the capture time,
  accurate to 1 millisecond, for every frame. This log is saved when the capture
  sequence is stopped.
  
  Two automatic termination methods are available in addition to the default
  method of manually stopping the capture sequence.
  
  
  ******** INSTALLATION ******** 
  
  1. Copy the AutoCollect.v, AutoCollect.dll, and Timers.dll files into the
     V++ executable files directory. This is usually found at the following
	 path: C:\Program Files\Digital Optics\V++\Executable
	 
  2. Start V++ and open the AutoCollect.v module file.
  
  3. Compile the module by pressing the Compile button on the toolbar (or you
     can select Compile from the Automation menu). You will see a new toolbar
	 appear and there will be some additional commands in the User menu.
	 
  4. Select Tools | Module List from the main menu and click on AutoCollect.v in
     the list.
	 
  5. Press the Startup button and select "Load and run this module at startup".
	   

  ******** UNINSTALLING ********
  
  To uninstall AutoCollect.v simply repeat step 5 above and select the "Do not
  automatically load this module at startup" option.  
   
}

toolbar 'Auto Collect' ;

const
  TimersLib = 'Timers.dll' ;
  AutoCollectLib = 'AutoCollect.dll' ;
  IniSection = 'AutoCollect' ;
 

{ Declare external routines }
function SetDdeTimer( Elapse:integer; Name:pointer ) : integer ;
external TimersLib ;

procedure KillDdeTimer( Id:integer ) ; 
external TimersLib ;

procedure KillAllTimers ; 
external TimersLib ;

function ShowSetupDialog : integer ;
external AutoCollectLib ;


var
  { general }
  TickId ;     // timer id for frame capture
  StopId ;	   // timer id for termination
  nFrame ;     // current frame number
  Count ;	   // total frames in this sequence
  Seq ;	  	   // sequence in memory
  
  { timing }
  Period ;     // milliseconds between frames
  nLimit ;     // frame number limit 
  tLimit ;	   // total time limit (ms)
  LogFile ;	   // set true if log file required

  { devices }
  PVCAM ;	   // true if PVCAM device
  nDevice ;    // selected device index
  ExpTime ;	   // exposure time 
  Shutter ;	   // true if using an external shutter

  { filing }
  ToDisk ;	   // true if saving each frame to disk
  sDir ;	   // default save folder (can be changed by user)
  sName ;      // base name for saved files
  LogName ;	   // base name of log file
  
  { display }
  Display ;    // display image 
  Ed ;	  	   // text output

  
  
// Forward declarations
procedure TimerSnap ; forward ;
procedure Initialize ; forward ;


function ZeroPad( s,n ) ;
{ Utility function for string formatting }

var
  i,z ;
  
begin
  z := '' ;
  for i := Length( s ) + 1 to n do  
    z := z + '0' ;
  ZeroPad := z + s ;
end;


function DateString ;
{ Utility function for date formatting }

var
  y,m,d ;
  Month ;

begin
  GetDate( y,m,d ) ;
  if m =  1 then Month := 'Jan' ;
  if m =  2 then Month := 'Feb' ;
  if m =  3 then Month := 'Mar' ;
  if m =  4 then Month := 'Apr' ;
  if m =  5 then Month := 'May' ;
  if m =  6 then Month := 'Jun' ;
  if m =  7 then Month := 'Jul' ;
  if m =  8 then Month := 'Aug' ;
  if m =  9 then Month := 'Sep' ;
  if m = 10 then Month := 'Oct' ;
  if m = 11 then Month := 'Nov' ;
  if m = 12 then Month := 'Dec' ;
  DateString := Str( d,'-',Month,'-',y ) ;
end;


procedure SelectDevice ;
{ Select the camera or frame grabber }

var
  dName ;
  
begin
  if PVCAM then begin
    dName := pvcGetCameraName( nDevice ) ;
	pvcOpenCamera( dName ) ;
    pvcSelectCamera( dName ) ;
  end
  else
    vidSelectDevice( nDevice ) ;
end;  { SelectDevice }


procedure SetupDisplayWindow ;
{ Create a frame display window }

var
  xs,ys ;

begin
  { determine output size }
  if not PVCAM then
    vidGetFrameInfo( xs,ys )
  else
    pvcGetCCDSize( xs,ys ) ;

  { set up display window }
  Display := CreateImage( typ_Byte,xs,ys ) ;
  Show( Display,'Display' ) ;
  SetWindowPos( Display,20,20 ) ;
    
end;  { SetupDisplayWindow }


procedure SetupLogFile ;
{ Create an editor to capture logging info }

const
  Width = 300 ;   // width of logging window
  Margin = 20 ;	  // distance from edge of desktop

var
  dx,dy ;
  
begin
  if LogFile and not IsEditor( Ed ) then begin
    GetDesktopSize( dx,dy ) ;
    Ed := CreateEditor( 'Log',dx-Width-Margin,Margin,dx-Margin,dy-Margin ) ;
    writeln( Ed,'Auto Collect log file' ) ;
	writeln( Ed,DateString ) ;
    writeln( Ed ) ;
  end;  
end;  { SetupLogFile }


function AcquireImage ;
{ Acquire one frame from the imaging device }

var
  xc,yc ;
  
begin
  if PVCAM then begin
    pvcGetCCDSize( xc,yc ) ;
	pvcSetExpTime( ExpTime ) ;
	AcquireImage := pvcCapture( 0,0,xc-1,yc-1 ) ;
  end
  else begin
    vidSetExposureTime( ExpTime ) ;
    AcquireImage := vidCapture ;
  end;	
end;  { AcquireImage } 


procedure ShowSetup ;
button 141,'Setup time lapse experiment' ;
menu 'Setup time lapse e&xperiment...' ;
{ Change the inter-frame period }

begin
  if ShowSetupDialog = id_Ok then
    Initialize ;
end;  { ShowSetup }


procedure Start ;
button 135,'Start acquiring frames' ;
menu '&Start acquiring frames' ;

begin
  if TickId = -1 then begin
  
    { init }
	SelectDevice ;
    SetDir( sDir ) ;
	SetupLogFile ;
	SetupDisplayWindow ;
	Count := 0 ;
	
	{ new sequence }
	if IsImage( Seq ) then Free( Seq ) ;
  
    { timed termination? }
	if tLimit > 0 then StopId := SetDdeTimer( tLimit,'TimerStop' ) ;

    { start timer }
    TickId := SetDdeTimer( Period,'TimerSnap' ) ;
	
	{ take the first image }
	TimerSnap ;
  
  end
  else
    WriteError( 'Timer is already running' ) ;
  
end;  { Start }


procedure Stop ;
button 157,'Stop acquiring frames' ;
menu 'S&top acquiring frames' ;
dde 'TimerStop' ;

begin
  if TickId >= 0 then begin
  
	{ kill termination timer }
	if StopId <> -1 then KillDdeTimer( StopId ) ;
	StopId := -1 ; 
  
    { kill main timer }
    KillDdeTimer( TickId ) ;
	TickId := -1 ;
	
    { clean up }
    WriteStatus( 'Frame capture stopped' ) ;
	writeln( Ed,'Stopped' ) ;
    if ToDisk then Save( Ed,LogName ) ;
    Hide( Display ) ;
	
	{ sequence in memory ? }
	if not ToDisk then Show( Seq ) ;
	
	{ indicate }
	WriteInfo( 'Auto collect capture stopped' ) ;

  end;
end;  { Stop }


procedure PreExposure ;
{ Perform pre-exposure tasks }

begin
end;  { PreExposure }


procedure PostExposure ;
{ Perform post-exposure tasks }

begin
end;  { PostExposure }


procedure TimerSnap ; dde 'TimerSnap' ; 
{ Capture an image }

var
  Name ;
  Image1 ;	 // captured frame 
  h,m,s,ms ;

begin
  { capture video image }
  GetTime( h,m,s,ms ) ;
  PreExposure ;
  Image1 := AcquireImage ;
  PostExposure ;
  Count := Count + 1 ;

  { update display }
  Display := Image1 ;
  WriteStatus( 'Frame: ',ZeroPad( Str( nFrame:1 ),4 ),'    Press Stop button to quit' ) ;
  
  { update log file }
  Name := Str( sName,ZeroPad( Str( nFrame:1 ),4 ) ) ;
  writeln( Ed,Name,chr(9),chr(9),ZeroPad( Str( h:1 ),2 ),':',ZeroPad( Str( m:1 ),2 ),':',ZeroPad( Str( s+(single(ms)/1000):1:3 ),6 ) ) ;

  { output }
  if ToDisk then
    Save( Image1,Str( Name,'.tif' ) )
  else
    if IsImage( Seq ) then
      AppendFrame( Seq,Image1 )
    else
      Seq := Image1 ; 
  
  { stop ? }
  if ( nLimit > 0 ) and ( Count >= nLimit ) then Stop ;
  
  { next }
  nFrame := nFrame + 1 ;
  
end;  { TimerSnap }


procedure SetDefaults ;
{ Set the default values for all parameters }

begin
  Period := 5000 ;
  ExpTime := 100 ;
  Shutter := true ;
  LogFile := true ;
  LogName := 'AutoCollect Log.txt' ;
  tLimit := 0 ;
  nLimit := 0 ;
  ToDisk := true ;
  sDir := 'C:\My Documents' ;
  sName := 'Frame' ;
  PVCAM := true ;
  nDevice := 0 ;
end;  { SetDefaults }


procedure Initialize ;
{ Read settings from the V++ INI file }

begin
  Period := val( ReadINIString( IniSection,'Period',Str( Period ) ) ) ;
  ExpTime := val( ReadINIString( IniSection,'ExpTime',Str( ExpTime ) ) ) ;
  Shutter := ( val( ReadINIString( IniSection,'UseShutter',Str( Shutter ) ) ) <> 0 ) ;
  LogFile := ( val( ReadINIString( IniSection,'UseLogFile',Str( LogFile ) ) ) <> 0 ) ;
  LogName := ReadINIString( IniSection,'LogFileName',LogName ) ;
  tLimit := val( ReadINIString( IniSection,'TimeLimit',Str( tLimit ) ) ) ;
  nLimit := val( ReadINIString( IniSection,'FrameLimit',Str( nLimit ) ) ) ;
  ToDisk := ( val( ReadINIString( IniSection,'DataToDisk',Str( ToDisk ) ) ) <> 0 ) ;
  sDir := ReadINIString( IniSection,'Folder',sDir ) ;
  sName := ReadINIString( IniSection,'BaseName',sName ) ;
  PVCAM := ( val( ReadINIString( IniSection,'UsePVCAM',Str( PVCAM ) ) ) <> 0 ) ;
  nDevice := val( ReadINIString( IniSection,'Device',Str( nDevice ) ) ) ;
end;  { Initialize }


procedure Finalize ; shutdown ;
{ Write settings to the V++ INI file }

begin
  WriteINIString( IniSection,'Period',Str( Period ) ) ;
  WriteINIString( IniSection,'ExpTime',Str( ExpTime ) ) ;
  WriteINIString( IniSection,'UseShutter',Str( Shutter ) ) ;
  WriteINIString( IniSection,'UseLogFile',Str( LogFile ) ) ;
  WriteINIString( IniSection,'LogFileName',LogName ) ;
  WriteINIString( IniSection,'TimeLimit',Str( tLimit ) ) ;
  WriteINIString( IniSection,'FrameLimit',Str( nLimit ) ) ;
  WriteINIString( IniSection,'DataToDisk',Str( ToDisk ) ) ;
  WriteINIString( IniSection,'Folder',sDir ) ;
  WriteINIString( IniSection,'BaseName',sName ) ;
  WriteINIString( IniSection,'UsePVCAM',Str( PVCAM ) ) ;
  WriteINIString( IniSection,'Device',Str( nDevice ) ) ;
end;  { Finalize }


begin
  SetDefaults ;
  Initialize ;  
  TickId := -1 ;
  StopId := -1 ;
  nFrame := 0 ;
  Count := 0 ;
  SetDialogWait( 15 ) ;
end.
